package cc.fascinated.bat.common;

import lombok.experimental.UtilityClass;
import net.dv8tion.jda.api.EmbedBuilder;

import java.time.LocalDateTime;

/**
 * @author Fascinated (fascinated7)
 */
@UtilityClass
public class EmbedUtils {
    /**
     * Builds a generic embed
     *
     * @return the embed builder
     */
    public static EmbedBuilder genericEmbed() {
        return new EmbedBuilder()
                .setTimestamp(LocalDateTime.now())
                .setColor(Colors.DEFAULT);
    }

    /**
     * Builds an error embed
     *
     * @return the embed builder
     */
    public static EmbedBuilder errorEmbed() {
        return new EmbedBuilder()
                .setTimestamp(LocalDateTime.now())
                .setColor(Colors.ERROR);
    }

    /**
     * Builds a success embed
     *
     * @return the embed builder
     */
    public static EmbedBuilder successEmbed() {
        return new EmbedBuilder()
                .setTimestamp(LocalDateTime.now())
                .setColor(Colors.SUCCESS);
    }

    /**
     * Builds a generic interaction error embed
     *
     * @param ex the exception
     * @return the embed builder
     */
    public static EmbedBuilder genericInteractionError(Exception ex) {
        return EmbedUtils.errorEmbed()
                .setDescription("An error occurred while processing your interaction. Please try again later.\n" +
                        "**Error:** ```java\n%s\n```" + ex.getLocalizedMessage());
    }
}
