package cc.fascinated.bat.common;

import java.awt.*;

/**
 * @author Fascinated (fascinated7)
 */
public class HexColorUtils {
    /**
     * Checks if the given string is a hex color
     *
     * @param hexColor the hex color to check
     * @return if the given string is a hex color
     */
    public static boolean isHexColor(String hexColor) {
        return hexColor.matches("^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$");
    }

    /**
     * Converts a hex color to a Color object
     *
     * @param hex the hex color to convert
     * @return the Color object
     */
    public static Color hexToColor(String hex) {
        if (hex == null || (!hex.matches("^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$"))) {
            throw new IllegalArgumentException("Invalid hex color code");
        }

        // Remove the '#' character
        hex = hex.substring(1);

        // If the hex code is 3 characters long, expand it to 6 characters
        if (hex.length() == 3) {
            char r = hex.charAt(0);
            char g = hex.charAt(1);
            char b = hex.charAt(2);
            hex = "" + r + r + g + g + b + b;
        }

        // Convert the hex string to an integer and create a Color object
        int rgb = Integer.parseInt(hex, 16);
        return new Color(rgb);
    }
}
