package cc.fascinated.bat.event;

import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.model.DiscordMessage;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberLeaderboardToken;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberPlayerScoreToken;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberScoreToken;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Invite;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.events.channel.ChannelCreateEvent;
import net.dv8tion.jda.api.events.channel.ChannelDeleteEvent;
import net.dv8tion.jda.api.events.channel.update.GenericChannelUpdateEvent;
import net.dv8tion.jda.api.events.guild.GuildBanEvent;
import net.dv8tion.jda.api.events.guild.GuildUnbanEvent;
import net.dv8tion.jda.api.events.guild.invite.GuildInviteCreateEvent;
import net.dv8tion.jda.api.events.guild.member.GuildMemberJoinEvent;
import net.dv8tion.jda.api.events.guild.member.GuildMemberRemoveEvent;
import net.dv8tion.jda.api.events.guild.member.GuildMemberRoleAddEvent;
import net.dv8tion.jda.api.events.guild.member.GuildMemberRoleRemoveEvent;
import net.dv8tion.jda.api.events.guild.member.update.GuildMemberUpdateBoostTimeEvent;
import net.dv8tion.jda.api.events.guild.member.update.GuildMemberUpdateNicknameEvent;
import net.dv8tion.jda.api.events.guild.member.update.GuildMemberUpdateTimeOutEvent;
import net.dv8tion.jda.api.events.guild.voice.GenericGuildVoiceEvent;
import net.dv8tion.jda.api.events.interaction.ModalInteractionEvent;
import net.dv8tion.jda.api.events.interaction.component.ButtonInteractionEvent;
import net.dv8tion.jda.api.events.interaction.component.StringSelectInteractionEvent;
import net.dv8tion.jda.api.events.message.MessageDeleteEvent;
import net.dv8tion.jda.api.events.message.MessageReceivedEvent;
import net.dv8tion.jda.api.events.message.MessageUpdateEvent;
import net.dv8tion.jda.api.events.user.update.UserUpdateAvatarEvent;
import net.dv8tion.jda.api.events.user.update.UserUpdateGlobalNameEvent;
import net.dv8tion.jda.api.events.user.update.UserUpdateNameEvent;

import java.time.OffsetDateTime;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
public interface EventListener {
    /**
     * Called when a ScoreSaber score is received
     *
     * @param score       the score that was set
     * @param leaderboard the leaderboard that the score was set on
     * @param player      the player that set the score
     */
    default void onScoresaberScoreReceived(@NonNull ScoreSaberPlayerScoreToken score, @NonNull ScoreSaberLeaderboardToken leaderboard,
                                           @NonNull ScoreSaberScoreToken.LeaderboardPlayerInfo player) {
    }

    /**
     * Called when a user joins a guild
     *
     * @param guild the guild the user joined
     * @param user  the user that joined the guild
     */
    default void onGuildMemberJoin(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull GuildMemberJoinEvent event) {
    }

    /**
     * Called when a user leaves a guild
     *
     * @param guild the guild the user left
     * @param user  the user that left the guild
     */
    default void onGuildMemberLeave(@NonNull BatGuild guild, BatUser user, @NonNull GuildMemberRemoveEvent event) {
    }

    /**
     * Called when a user types a message
     *
     * @param guild the guild that the message was sent in
     * @param user  the user that sent the message
     */
    default void onGuildMessageReceive(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull MessageReceivedEvent event) {
    }

    /**
     * Called when a user updates a message
     *
     * @param guild the guild that the message was updated in
     * @param user  the user that updated the message
     */
    default void onGuildMessageEdit(@NonNull BatGuild guild, @NonNull BatUser user, DiscordMessage oldMessage,
                                    @NonNull DiscordMessage newMessage, @NonNull MessageUpdateEvent event) {
    }

    /**
     * Called when a user deletes a message
     *
     * @param guild the guild that the message was deleted in
     * @param user  the user that deleted the message
     */
    default void onGuildMessageDelete(@NonNull BatGuild guild, BatUser user, DiscordMessage message, @NonNull MessageDeleteEvent event) {
    }

    /**
     * Called when a user selects a string
     *
     * @param guild the guild that the string was selected in
     * @param user  the user that selected the string
     */
    default void onStringSelectInteraction(BatGuild guild, @NonNull BatUser user, @NonNull StringSelectInteractionEvent event) {
    }

    /**
     * Called when a user interacts with a button
     *
     * @param guild the guild that the button was interacted with in
     * @param user  the user that interacted with the button
     */
    default void onButtonInteraction(BatGuild guild, @NonNull BatUser user, @NonNull ButtonInteractionEvent event) {
    }

    /**
     * Called when a user interacts with a modal
     *
     * @param guild the guild that the modal was interacted with in
     * @param user  the user that interacted with the modal
     */
    default void onModalInteraction(BatGuild guild, @NonNull BatUser user, @NonNull ModalInteractionEvent event) {
    }

    /**
     * Called when a user updates their global name
     *
     * @param user    the user that updated their global name
     * @param oldName the old global name
     * @param newName the new global name
     */
    default void onUserUpdateGlobalName(@NonNull BatUser user, String oldName, String newName, @NonNull UserUpdateGlobalNameEvent event) {
    }

    /**
     * Called when a user updates their nickname in a guild
     *
     * @param guild   the guild that the user updated their nickname in
     * @param user    the user that updated their nickname
     * @param oldName the old nickname
     * @param newName the new nickname
     */
    default void onGuildMemberUpdateNickname(@NonNull BatGuild guild, @NonNull BatUser user, String oldName, String newName, @NonNull GuildMemberUpdateNicknameEvent event) {
    }

    /**
     * Called when a user gets roles added to them
     *
     * @param guild      the guild that the user added the role in
     * @param user       the user that added the role
     * @param rolesAdded the roles that were added
     */
    default void onGuildMemberRoleAdd(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull List<Role> rolesAdded, @NonNull GuildMemberRoleAddEvent event) {
    }

    /**
     * Called when a user gets roles removed from them
     *
     * @param guild      the guild that the user removed the role in
     * @param user       the user that removed the role
     * @param rolesAdded the roles that were removed
     */
    default void onGuildMemberRoleRemove(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull List<Role> rolesAdded, @NonNull GuildMemberRoleRemoveEvent event) {
    }

    /**
     * Called when a channel is created
     *
     * @param guild the guild that the channel was created in
     */
    default void onChannelCreate(@NonNull BatGuild guild, @NonNull ChannelCreateEvent event) {
    }

    /**
     * Called when a channel is deleted
     *
     * @param guild the guild that the channel was deleted in
     */
    default void onChannelDelete(@NonNull BatGuild guild, @NonNull ChannelDeleteEvent event) {
    }

    /**
     * Called when a user is banned from a guild
     *
     * @param guild the guild that the user was banned from
     * @param user  the user that was banned
     */
    default void onGuildMemberBan(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull GuildBanEvent event) {
    }

    /**
     * Called when a user is unbanned from a guild
     *
     * @param guild the guild that the user was unbanned from
     * @param user  the user that was unbanned
     */
    default void onGuildMemberUnban(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull GuildUnbanEvent event) {
    }

    /**
     * Called when a user gets timed out in a guild (gets muted)
     *
     * @param guild the guild that the user timed out in
     * @param user  the user that timed out
     */
    default void onGuildMemberTimeout(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull GuildMemberUpdateTimeOutEvent event) {
    }

    /**
     * Called when a channels state is updated
     *
     * @param guild the guild that the channel was updated in
     * @param event the event that was fired
     */
    default void onGenericChannelUpdate(@NonNull BatGuild guild, @NonNull GenericChannelUpdateEvent<?> event) {
    }

    /**
     * Called when a user updates their username
     *
     * @param user    the user that updated their name
     * @param oldName the old username
     * @param newName the new username
     */
    default void onUserUpdateName(@NonNull BatUser user, String oldName, String newName, @NonNull UserUpdateNameEvent event) {
    }

    /**
     * Called when a user updates their avatar
     *
     * @param user         the user that updated their avatar
     * @param oldAvatarUrl the old avatar url
     * @param newAvatarUrl the new avatar url
     */
    default void onUserUpdateAvatar(@NonNull BatUser user, String oldAvatarUrl, String newAvatarUrl, @NonNull UserUpdateAvatarEvent event) {
    }

    /**
     * Called when a user joins or leaves a voice channel
     *
     * @param guild the guild that the user joined or left the voice channel in
     * @param user  the user that joined or left the voice channel
     */
    default void onGuildVoiceUpdate(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull GenericGuildVoiceEvent event) {
    }

    /**
     * Called when a guild invite is created
     *
     * @param guild  the guild that the invite was created in
     * @param invite the invite that was created
     */
    default void onGuildInviteCreate(@NonNull BatGuild guild, @NonNull Invite invite, @NonNull GuildInviteCreateEvent event) {
    }

    /**
     * Called when a user boosts a guild
     *
     * @param guild        the guild that the user boosted
     * @param user         the user that boosted the guild
     * @param oldBoostTime the old boost time
     * @param newBoostTime the new boost time
     * @param event        the event that was fired
     */
    default void onGuildMemberBoostAdd(@NonNull BatGuild guild, @NonNull BatUser user, OffsetDateTime oldBoostTime,
                                       @NonNull OffsetDateTime newBoostTime, @NonNull GuildMemberUpdateBoostTimeEvent event) {
    }

    /**
     * Called when a user boost expires in a guild
     *
     * @param guild        the guild that the user's boost expired in
     * @param user         the user that the boost expired for
     * @param oldBoostTime the old boost time
     * @param newBoostTime the new boost time
     * @param event        the event that was fired
     */
    default void onGuildMemberBoostRemove(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull OffsetDateTime oldBoostTime,
                                       OffsetDateTime newBoostTime, @NonNull GuildMemberUpdateBoostTimeEvent event) {
    }

    /**
     * Called when Bat is shutting down
     */
    default void onShutdown() {
    }
}
