package cc.fascinated.bat.features.logging.command;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.logging.LogCategory;
import cc.fascinated.bat.features.logging.LogProfile;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Component("logs:set.sub")
@CommandInfo(name = "set", description = "Set the channel for a log type")
public class SetSubCommand extends BatSubCommand {
    public SetSubCommand() {
        super.addOption(OptionType.STRING, "type", "The type of log to set", true);
        super.addOption(OptionType.CHANNEL, "channel", "The channel to set the log to", true);
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        OptionMapping typeOption = event.getOption("type");
        if (typeOption == null) {
            return;
        }
        OptionMapping channelOption = event.getOption("channel");
        if (channelOption == null) {
            return;
        }
        String type = typeOption.getAsString();
        TextChannel targetChannel = channelOption.getAsChannel().asTextChannel();
        LogProfile profile = guild.getLogProfile();

        // Set the log channel for all log types
        if (type.equalsIgnoreCase("all")) {
            EmbedDescriptionBuilder description = new EmbedDescriptionBuilder("Log Channel");
            description.appendLine("Successfully set the log channel for all log types to %s".formatted(targetChannel.getAsMention()), false);
            description.emptyLine();
            for (int i = 0; i < LogCategory.values().length; i++) {
                LogCategory category = LogCategory.values()[i];
                if (i != 0) {
                    description.emptyLine();
                }
                description.appendLine("**__%s__**".formatted(category.getName()), false);
                for (LogType logType : LogType.getLogTypesByCategory(category.getName())) {
                    description.appendLine(logType.getName(), true);
                    profile.setLogChannel(logType, targetChannel);
                }
            }

            event.replyEmbeds(EmbedUtils.successEmbed()
                    .setDescription(description.build())
                    .build()).queue();
            return;
        }

        // Set the log channel for a specific log category
        LogCategory logCategory = LogCategory.getLogCategory(type);
        if (logCategory != null) {
            List<LogType> category = LogType.getLogTypesByCategory(type);
            EmbedDescriptionBuilder description = new EmbedDescriptionBuilder("Log Channel");
            description.appendLine("Successfully set the log channel for the `%s` category to %s"
                    .formatted(logCategory.getName(), targetChannel.getAsMention()), false);
            description.emptyLine();
            for (LogType logType : category) {
                description.appendLine(logType.getName(), true);
                profile.setLogChannel(logType, targetChannel);
            }

            event.replyEmbeds(EmbedUtils.successEmbed()
                    .setDescription(description.build())
                    .build()).queue();
            return;
        }

        // Set the log channel for a specific log type
        LogType logType = LogType.getLogType(type);
        if (logType == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("Invalid log type")
                    .build()).queue();
            return;
        }

        profile.setLogChannel(logType, targetChannel);
        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("Successfully set the log channel for `%s` to %s".formatted(logType.getName(), targetChannel.getAsMention()))
                .build()).queue();
    }
}
