package cc.fascinated.bat.features.logging.listeners;

import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.logging.LogFeature;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.entities.Invite;
import net.dv8tion.jda.api.events.guild.invite.GuildInviteCreateEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2
public class GuildListener implements EventListener {
    private final LogFeature logFeature;

    @Autowired
    public GuildListener(@NonNull ApplicationContext context) {
        this.logFeature = context.getBean(LogFeature.class);
    }

    @Override
    public void onGuildInviteCreate(@NonNull BatGuild guild, @NonNull Invite invite, @NonNull GuildInviteCreateEvent event) {
        log.info("Invite created in guild \"{}\" with code \"{}\"", guild.getGuild().getName(), invite.getCode());

        guild.getDiscordGuild().retrieveInvites().queue(invites -> {
            Invite newInvite = invites.stream().filter(i -> i.getCode().equals(invite.getCode())).findFirst().orElse(null);
            if (newInvite == null) {
                return;
            }
            newInvite.expand().queue(expanded -> logFeature.sendLog(guild, LogType.INVITE_CREATE, EmbedUtils.successEmbed()
                    .setDescription(new EmbedDescriptionBuilder("Invite Created")
                            .appendLine("Invite: %s".formatted(expanded.getUrl()), true)
                            .appendLine("Channel: %s".formatted(expanded.getChannel()), true)
                            .appendLine("Max Uses: %s".formatted(expanded.getMaxUses() == 0 ? "Infinite" : expanded.getMaxUses()), true)
                            .appendLine("Creator: %s".formatted(expanded.getInviter() == null ? "Unknown" : expanded.getInviter().getAsMention()), true)
                            .build())
                    .build()));
        });
    }
}
