package cc.fascinated.bat.features.welcomer.command;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.welcomer.WelcomerPlaceholders;
import cc.fascinated.bat.features.welcomer.WelcomerProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("welcomer:current.sub")
@CommandInfo(name = "current", description = "View the current welcomer configuration")
public class CurrentSubCommand extends BatSubCommand {
    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        WelcomerProfile profile = guild.getWelcomerProfile();
        if (!profile.isEmbed() && !profile.isMessage()) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The welcomer is not configured.\n\n" + getPlaceholders(guild, user))
                    .build()).queue();
            return;
        }

        if (profile.isEmbed()) {
            event.replyEmbeds(profile.getWelcomerEmbed().buildEmbed(guild, user)
                    .appendDescription("\n\n" + getPlaceholders(guild, user))
                    .build()).queue();
            return;
        }
        if (profile.isMessage()) {
            event.replyEmbeds(EmbedUtils.genericEmbed()
                    .setDescription("**Preview:** %s\n*note: the real message won't be an embed*\n\n%s".formatted(
                            profile.getWelcomerMessage().buildMessage(guild, user),
                            getPlaceholders(guild, user)
                    ))
                    .build()).queue();
        }
    }

    /**
     * Get the placeholders that the user can use
     *
     * @param replacements What to replace the placeholders using
     * @return The placeholders
     */
    public String getPlaceholders(Object... replacements) {
        StringBuilder builder = new StringBuilder();
        builder.append("**Available Placeholders:**\n");
        for (WelcomerPlaceholders placeholder : WelcomerPlaceholders.values()) {
            String placeholderString = placeholder.getPlaceholder();
            builder.append("`").append(placeholderString).append("` - ")
                    .append(WelcomerPlaceholders.replaceAllPlaceholders(placeholderString, replacements)).append("\n");
        }
        return builder.toString();
    }
}
